/*****************************************************
     

Project : CW    
Version : 1.1
Date    : 19.06.2018
Author  : Toni                            
Company : rtk                             
Comments: 
		      
		       1101
		      GDO0   
		  Asynchronous Serial Operation
		   ,    BMP180 (BMP085)
		       .
		       3580 
		    14318 .


Chip type           : ATmega328p
Program type        : Application
Clock frequency     : 16,0 MHz
*****************************************************/


#include <stdio.h>
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <math.h>
#include <avr/pgmspace.h>

#include "atmega328p.h"
#include "uart.h"
#include "cc1101.h"
#include "main.h"
#include "morze.h"
#include "adc.h"
#include "atmega_bmp180_lib.h"


	int32_t temperature = 0;
	int32_t pressure = 0;
	int16_t BMP180_calibration_int16_t[8];
	int16_t BMP180_calibration_uint16_t[3];
	uint8_t error_code=0;
	uint8_t str_error_code[35];
	uint8_t str_altitude[35];
	int32_t altitude;


// 
//=================================================================
int main(void)
	{
	atmega328_init();  // 
	morze_init();  //  

#ifdef HF_CARRIER_TEST	
	hf_carrier_on();
	while(1);
#endif
	
	flags.low_voltage_no_telemetry=0;
	
	BMP180_OFF;
	//  TWI
	i2cSetBitrate(1000);
	//  BMP085
//	BMP180_Calibration(BMP180_calibration_int16_t, BMP180_calibration_uint16_t,&error_code);


	// 
	
	ccxxx0_PowerOnReset();
	ccxxx0_Setup(&rfSettings);
	ccxxx0_Strobe(CCxxx0_SIDLE);
	ccxxx0_WriteBurst(CCxxx0_PATABLE, &paTable[TENDBM], 1); // max power
	_delay_ms(1);
	
	while(1)
		{
		u_measure();

		if (flags.low_voltage_no_telemetry==0)  //       U_POROG_NO_TELEMETRY   
			{
			#ifndef DEBUG_ON
			cw_tx(cw_str);
			cw_tx(cw_str_vbatt);
			#endif
			
			if (flags.low_voltage==0)
				{
				// .       ,  ,   TEMP ERR
				if (temp_measure() == 0)
					{
					#ifndef DEBUG_ON
					cw_tx(cw_str_temp);
					cw_tx(str_altitude);
						#else
						uart_tx_string(str_altitude);
						uart_tx_string(cw_str_vbatt);
						uart_tx_string(str_error_code);
						uart_tx_string(cw_str_temp);
						uart_tx_string(cw_str_press);
					
						_delay_ms(1000);
					#endif
					
					}
					else
						{
						#ifndef DEBUG_ON
						cw_tx((unsigned char *)"TEMP ER");
							#else
							uart_tx_string((unsigned char *)"Temp sensor error");
						#endif
						}
						
				
				}
			#ifndef DEBUG_ON
			cw_tx(cw_str_end);
			#endif

			}
			else
				{
					uart_tx_string((unsigned char *)"Low batt voltage");
				}
			#ifndef DEBUG_ON
			beeper(PERIOD_BEEP,(unsigned char *)"B");   //    .   S  BEEP_PAUSE 
			#endif
		}
		
	}

//==================================================================



//---------------FUNCTIONS--------------------------

//          
//  quantity,    position, 
//    digit     (digit    const uint32_t  pow10Table32[]= ) 
// digit=1   , digit=2   , digit=3     ...
void inttosymb_cycle_sub(int32_t value, uint8_t *buffer, uint8_t position, uint8_t quantity, uint8_t digit)
	{
	digit=10-digit;
//	if (quantity>digit) return;  // , quantity<=digit  !!
//	buffer+= position;
	for (uint8_t i=0;i<position;i++) buffer++;
 	for (uint8_t i=0;i<quantity;i++)
		{
  	    uint32_t pow10 = pow10Table32[digit++];
 	    uint8_t count = 0;
  	    while(value >= pow10)
			{
			count ++;
			value -= pow10;
			}
  	    *buffer = count + '0';
		buffer++;
		}
	}



void u_measure(void)  //  
		{
		uint32_t voltage=0;
		
		voltage=getVoltage(); // 
		
		if (voltage<U_POROG) flags.low_voltage=1;
			else flags.low_voltage=0;
			
		if (voltage<U_POROG_NO_TELEMETRY) flags.low_voltage_no_telemetry=1;
			else flags.low_voltage_no_telemetry=0;
			
		if (flags.low_voltage_min_pow == 0)
			{
			if (voltage < U_POROG_MIN_POW)   //     ,    7 dBm
				{
				flags.low_voltage_min_pow=1;
				ccxxx0_WriteBurst(CCxxx0_PATABLE, &paTable[SEVENDBM], 1);
				}
			}
			
		//      
		inttosymb_cycle_sub(voltage, cw_str_vbatt, 7, 3, 4); //(, , ,    ,     )

		}



	uint8_t temp_measure(void)  //   0,    1
		{
		char *znak;
		BMP180_ON;
		_delay_ms(5);
		BMP180_Calibration(BMP180_calibration_int16_t, BMP180_calibration_uint16_t,&error_code);
		_delay_ms(5);
		bmp180Convert(BMP180_calibration_int16_t, BMP180_calibration_uint16_t,&temperature, &pressure,&error_code);
		BMP180_OFF;
		
		snprintf_P((char *)str_error_code, 20, PSTR("ERROR CODE %d \r"), (int)error_code);
		if (error_code > 0)
			{
				return 1;
			}
			
	//	snprintf((char *)cw_str_press, 20, "PRESS %ld \r", pressure);

		float temp;
		temp =  (float) pressure / 101325;
		temp = 1-pow(temp, 0.19029);
		altitude = 44330*temp;
		snprintf_P((char *)str_altitude, 20, PSTR(" ALT %d"), (int) altitude);
		snprintf_P((char *)cw_str_temp, 20, PSTR(" TEMP %+d"), (int) temperature);
		
		znak=strchr((const char*)cw_str_temp, (int)'+');
			if (znak != NULL)
				{
				*znak='P';
				}
			else
				{
				znak=strchr((const char*)cw_str_temp, (int)'-');
					if (znak != NULL)
					{
					*znak='M';
					}
				}
	
		_delay_ms(20);
		return 0;
		}


void beeper (unsigned char i, unsigned char *symb)
	{
	while (i--)
		{
		ccxxx0_Strobe(CCxxx0_SPWD);
		_delay_ms(BEEP_PAUSE);
		ccxxx0_Strobe(CCxxx0_SIDLE);
		_delay_us(300);
		cw_tx(symb);
		}
	}

